// AYUCR Camera Controller

include "PocketGuiLib.h"
include "PocketGui.h"
include "SerialLib.h"

#define VERSION "2.1"

//------------------------------------------------------------------------
@cid	"AYUc";
@name	"AYUCR Camera Controller";
@dbname	"AYUCR Camera Controller";
@ver	VERSION;
@licon1	"large.bmp";
@sicon1 "small.bmp";

//------------------------------------------------------------------------

#define PROGLEN 16
#define PROGUSED 8
int proglen = 8; // default to old size
int prog[PROGLEN];
string recvstr;

//------------------------------------------------------------------------

#define PROGBUT 40
#define GETBUT 41
#define SAVEBUT 42
#define LOADBUT 43
#define TESTBUT 44
#define VERBUT 45

create_controls()
{
	int i=1;
	GCount (i++,"",21,0,5);
	GCount (i++,"",21,3,5);
	GCount (i++,"",21,6,5);
	GCount (i++,"",21,8,5);
	GCount (i++,"",21,10,5);
	GCount (i++,"",21,12,5);
	GCount (i++,"",21,15,5);
	GCount (i++,"",21,17,5);

	GLabel(i++,"Launch Detect (1/100 sec)",1,0,20);
	GLabel(i++,"Launch Delay (1/10 sec)",1,3,20);
	GLabel(i++,"Number of Groups",1,6,20);
	GLabel(i++,"Pictures per Group",1,8,20);
	GLabel(i++,"Picture Interval (1/10 sec)",1,10,20);
	GLabel(i++,"Group Interval (sec)",1,12,20);
	GLabel(i++,"Shutter Speed (1/10 sec)",1,15,20);
	GLabel(i++,"Mode",1,17,20);

	GText(VERBUT,"xxx",1,19,20);

	GButton (PROGBUT,"Prog",1,22,5);
	GButton (GETBUT,"Read",6,22,5);
	GButton (LOADBUT,"Load",11,22,5);
	GButton (SAVEBUT,"Save",16,22,5);
	GButton (TESTBUT,"Test",21,22,5);
}

//------------------------------------------------------------------------

save_program(string name)
{
    int i;

    dbcreate(name);

    for(i=0; i<PROGLEN; i++)
        dbwrite(prog[i]);

    dbclose();
}

load_program(string name)
{
    int i;

    dbopen(name);

    for(i=0; i<PROGLEN; i++)
        prog[i] = dbread('i');

    dbclose();
}

read_fields()
{
     int i;

     for(i=0; i<PROGUSED; i++)
         prog[i] = GS[i+1];
}

load_fields()
{
     int i;
     string s;

    for(i=0; i<PROGUSED; i++) {
        GSet(i+1, prog[i]);
    }
}

send_program ()
{
    int i;
    char ack[3];

    read_fields();

    seropen (2400, "8N1X",1000);

    tone (1000,25);
    tone (500,50);

    sersendstr ("/P");
    sleep (600);

    sersendarray(prog, proglen);
    serrecvarray(ack,3);
    serclose ();

    if (ctostr (ack) == "OK")
        alert ("Programming sucessful");
    else
        alert ("Programming failed -" + ctostr (ack));
}

recv_program ()
{
    int i, c;
    char ack[3];

    seropen (2400, "8N1X",100);

    tone (500,50);
    tone (1000, 25);

    // Send the dump command
    sersendstr ("/D");

    for (i = 0; i < proglen; i++) {
        c=serrecv();
        if(c>255)
            break;
       
        prog[i]=c;
    }

    serrecvarray(ack,3);
    serclose();

    if (i<proglen || ctostr (ack) != "OK"){
        alert ("Dump failed reading " + i + " bytes");
        read_fields();
        return;
     }

    load_fields();
}

test_shutter ()
{
    char ack[3];
    seropen (2400, "8N1X",500);

    // Send the command
    sersendstr ("/S");

    serrecvarray(ack,3);
    serclose();

    if(ctostr (ack) != "OK"){
        alert ("Comm Error");
    }
}

get_version()
{
    int i, c;
    char str[12];
    string ver;

    seropen (2400, "8N1X",100);

    // Send the command
    sersendstr ("/V");

    serrecvarray(str,12);
    serclose();

    ver = ctostr(str);
    for(i=0;i<12;i++){
        if(substr(ver,i,1)=="V"){
            GSet(VERBUT, substr(ver,i,5));
            return;
        }
    }

    GSet(VERBUT, "???");
}

get_proglen()
{
    char str[12];
    string size;

    seropen (2400, "8N1X",100);

    // Send the command
    sersendstr("/Z");

    serrecvarray(str,12);
    serclose();

    size=ctostr(str);
    if(strright(size,2)=="OK") {
        proglen=str[0];
        tone(1500, 100);
    }
    else {
        tone(500, 100);
    }
}

main()
{
	int e;

    GInit("AYUCR Camera Controller " + VERSION);
    create_controls();

    load_program("AYUCR");
    load_fields();

     // Get version and set proglen
    get_version();
    get_proglen();

    GRefresh();

    do{
        e = GEvent();

        if(e == PROGBUT) {
            send_program();
        }
        else if(e == GETBUT) {
            recv_program();
        }
        else if(e == SAVEBUT) {
           read_fields();
           save_program("AYUCR");
           tone (1000, 25);
       }
       else if(e == LOADBUT) {
           load_program("AYUCR");
           load_fields();
           tone(1500, 25);
       }
       else if(e == TESTBUT) {
           tone(1500, 25);
           test_shutter();
       }
    } while (1);
}
